-- --------------------------------------------------------------------------------
-- Routine DDL
-- Note: comments before and after the routine body will not be stored by the server
-- --------------------------------------------------------------------------------
DELIMITER $$

CREATE DEFINER=`grp20`@`%` PROCEDURE `sp_createRandomData`()
BEGIN
	-- define general vars
	DECLARE nbrOfOrders INT DEFAULT FLOOR(1 + RAND() * 10);
	DECLARE nbrOfProductsPerOrder INT DEFAULT FLOOR(1 + RAND() * 20);

	DECLARE idProduct INT;
	DECLARE idOrder INT;
	DECLARE cntOrder INT;

	-- product
	DECLARE v_productname VARCHAR(50);
	DECLARE v_price FLOAT;
	DECLARE v_amount INT;
	DECLARE v_amountmin INT;
	DECLARE v_active BIT;

	-- order
	DECLARE v_number INT;
	DECLARE v_paystate INT;
	DECLARE v_orderstate INT;
	DECLARE v_orderdate DATE;
	DECLARE v_orderdatenew DATE;
	DECLARE v_deliveraddress INT;
	DECLARE v_deliverstate INT;
	DECLARE v_deliverdate DATE;
	DECLARE v_delivercost FLOAT;
	DECLARE v_customerid INT;
	DECLARE v_salesid INT;

	-- orderposition
	DECLARE v_position INT;
	DECLARE v_productid INT;
	-- DECLARE v_amount INT; -- product
	DECLARE v_amountstock INT;
	-- DECLARE v_price FLOAT; -- product
	DECLARE v_outstanding BIT;

	-- stockorder / stock
	DECLARE v_stockordertyp INT;
	DECLARE v_stockorderstate INT;
	DECLARE v_stockdeliverdate DATE;
	-- DECLARE v_amountstock INT; -- orderposition
	DECLARE v_amountstockmin INT;
	DECLARE v_stockorderamount INT;
	DECLARE v_stockordermaxdeliverdate DATE;
	DECLARE v_stocknextdeliverdate DATE;
	DECLARE v_finaldeliverdate DATE;

	-- bill
	DECLARE v_billaddress INT;
	DECLARE v_billstate INT;
	DECLARE v_billtyp INT;
	DECLARE v_invoicedate DATE;
	DECLARE v_paymentduedate DATE;
	DECLARE v_dunningcosts FLOAT;


	-- ----- DELETE EXISTING DATA --------------------------------------------------------------------------
	SET FOREIGN_KEY_CHECKS = 0; -- Disable foreign key checking.
	TRUNCATE TABLE grp20.stockorder;
	TRUNCATE TABLE grp20.bill;
	TRUNCATE TABLE grp20.orderposition;
	TRUNCATE TABLE grp20.orderitem;
	TRUNCATE TABLE grp20.product;
	SET FOREIGN_KEY_CHECKS = 1; -- Enable foreign key checking.
	-- ----- DELETE EXISTING DATA --------------------------------------------------------------------------


	-- ----- CREATE PRODUCTS ------------------------------------------------------------------------------
	-- Create 2'000 products (10'000 - 11'999 | active and inactice) 
	SET idProduct = 10000;
	WHILE idProduct < 12000 DO
		SET v_productname = concat('Product ', cast(idProduct as char(14)));
		SET v_price       = RAND() * 100;
		SET v_amountmin   = FLOOR(RAND() * 20);
		SET v_amount      = RAND() * 50 + v_amountmin;
		SET v_active      = FLOOR(RAND() * 2);

		IF (v_active = 1) THEN
			-- active product
			INSERT INTO grp20.product (`number`, productname, price, amount, amountmin, active)
				VALUES (idProduct, v_productname, v_price, v_amount, v_amountmin, v_active);
		ELSE
			-- inactive product
			INSERT INTO grp20.product (`number`, productname, price, amount, amountmin, active)
				VALUES (idProduct, v_productname, 0, 0, 0, v_active);
		END IF;

		SET idProduct = idProduct + 1;
	END WHILE;
	-- ----- CREATE PRODUCTS ------------------------------------------------------------------------------


	-- ----- CREATE ORDERS --------------------------------------------------------------------------------
	SET v_paystate       = 1;
	SET v_orderstate     = 1;
	SET v_deliveraddress = 3;
	SET v_deliverstate   = 1;
	SET cntOrder         = 1;
	SET v_orderdatenew   = ADDDATE(DATE(NOW()), INTERVAL -(RAND() * 20)  DAY); -- Start: Today - random 20days

	-- Create 1..100 Orders
	WHILE cntOrder <= nbrOfOrders DO
		SELECT orderdate
			INTO v_orderdatenew
			FROM grp20.orderitem
			ORDER BY orderdate DESC
			LIMIT 1;

		SET v_orderdate   = ADDDATE(DATE(v_orderdatenew), INTERVAL (RAND() * 3)  DAY); 
		SET v_customerid  = FLOOR(6 + RAND() * 2); -- customer id 6..7
		SET v_number      = cntOrder + 100;
		SET v_deliverdate = ADDDATE(v_orderdate, INTERVAL 1 DAY);  -- deliver next day (also on Weekend) -- ADDDATE(DATE(NOW()), INTERVAL (RAND() * 20)-9  DAY); 
		SET v_delivercost = RAND() * 20;
		SET v_salesid     = FLOOR(1 + RAND() * 3); -- sales id 1..3

		INSERT INTO grp20.orderitem (number, paystate, orderstate, orderdate, deliveraddress, deliverstate, deliverdate, delivercost, customerid, salesid)
			VALUES (v_number, v_paystate, v_orderstate, v_orderdate, v_deliveraddress, v_deliverstate, v_deliverdate, v_delivercost, v_customerid, v_salesid);
	
		-- --------------------------------------------------------------------------------------------------------
		-- add 1..100 products to this order (orderposition)
		SET idOrder       = LAST_INSERT_ID();
		SET v_position    = 1;

		WHILE v_position <= nbrOfProductsPerOrder DO
			-- use only existing and active products
			SELECT id, price, amount, amountmin
				INTO v_productid, v_price, v_amountstock, v_amountstockmin
				FROM grp20.product
				WHERE id >= (SELECT FLOOR( MAX(id) * RAND()) FROM grp20.product) AND active = 1 AND id NOT IN (SELECT productid FROM grp20.orderposition WHERE orderid = idOrder)
				ORDER BY id
				LIMIT 1;
			
			SET v_amount = FLOOR(1 + RAND() * 20);
			IF (v_amount <= v_amountstock) THEN
				SET v_outstanding = 0;
			ELSE
				SET v_outstanding = 1;
			END IF;

			
			-- --------------------------------------------------------------------------------------------------------
			-- do we need to create a stock order?
			SET v_finaldeliverdate = v_deliverdate;
			IF (v_amount > (v_amountstock - v_amountstockmin)) THEN
				-- create stockorder
				SET v_stocknextdeliverdate = v_finaldeliverdate;
				-- get the latest stock deliver date for this product
				SELECT deliverdate
					INTO v_stockordermaxdeliverdate
					FROM grp20.stockorder
					WHERE productid = v_productid
					ORDER BY deliverdate DESC
					LIMIT 1;
				
				IF (v_stockordermaxdeliverdate > v_stocknextdeliverdate) THEN
					SET v_stocknextdeliverdate = v_stockordermaxdeliverdate;
				END IF;
				SET v_stocknextdeliverdate = ADDDATE(v_stocknextdeliverdate, INTERVAL (RAND() * 5) DAY); -- orderdate + random
				IF (v_stocknextdeliverdate >= v_finaldeliverdate) THEN
					SET v_finaldeliverdate = ADDDATE(v_stocknextdeliverdate, INTERVAL 1 DAY);
				END IF;

				SET v_stockordertyp   = 1;
				SET v_stockorderstate = 1;
				SET v_stockorderamount = CEIL((v_amount - v_amountstock + CEIL(v_amountstockmin/10)*20)/10)*10; --  order twice the minimum and ceil to next 10x amount

				INSERT INTO grp20.stockorder (productid, amount, state, typ, deliverdate, orderdate)
					VALUES (v_productid, v_stockorderamount, v_stockorderstate, v_stockordertyp, v_stocknextdeliverdate, v_orderdate);

				-- do we need to update deliver date in order?
				IF (v_amount > v_amountstock) THEN
					UPDATE grp20.orderitem SET deliverdate = v_finaldeliverdate, deliverstate = 2 WHERE id = idOrder AND deliverdate < v_finaldeliverdate;
				END IF;
			END IF; -- do we need to create a stock order?
		
			INSERT INTO grp20.orderposition (orderid, position, productid, amount, price, outstanding)
				VALUES (idOrder, v_position, v_productid, v_amount, v_price, v_outstanding);

			SET v_position = v_position + 1;
		END WHILE; -- add products to the order

		-- --------------------------------------------------------------------------------------------------------
		-- add Bill
		SELECT deliverdate INTO v_finaldeliverdate FROM grp20.orderitem WHERE id = idOrder;
		SET v_billaddress    = 2;
		SET v_billstate      = 1;
		SET v_billtyp        = 1;
		SET v_invoicedate    = v_finaldeliverdate; -- deliverdate
		SET v_paymentduedate = ADDDATE(v_finaldeliverdate, INTERVAL 10 DAY); -- deliverdate + 10day
		SET v_dunningcosts   = 0;

		INSERT INTO grp20.bill (orderid, billaddress, state, typ, invoicedate, paymentduedate, dunningcosts)
			VALUES (idOrder, v_billaddress, v_billstate, v_billtyp, v_invoicedate, v_paymentduedate, v_dunningcosts);


		SET cntOrder = cntOrder + 1;
	END WHILE;

	-- ----- CREATE ORDERS --------------------------------------------------------------------------------

END