package ch.hslu.appe.order.persistency;

import java.util.List;

import ch.hslu.appe.order.model.Order;

/**
 * Order Persistency Interface.
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 */
public interface OrderPersistency {
    /**
     * Creates a new Order.
     * @param order new Order to be created
     * @return true if success otherwise false
     */
    boolean createOrder(Order order);

    /**
     * Deletes a Order.
     * @param orderId Order-ID to be deleted
     * @return true if success otherwise false
     */
    boolean deleteOrder(int orderId);

    /**
     * Deletes a Order based on OrderNumber.
     * @param orderNumber orderNumber to be deleted
     * @return true if success otherwise false
     */
    boolean deleteOrderByNumber(int orderNumber);

    /**
     * Returns the next free OrderNumber.
     * @return next free OrderNumber
     */
    int getNextOrderNumber();

    /**
     * Returns a specific Order.
     * @param orderNumber The Order-Number
     * @return Returns the requested Order, or Null if failed / wrong id
     */
    Order getOrder(int orderNumber);

    /**
     * Returns all Orders.
     * @return Returns a List with all Orders
     */
    List<Order> getOrders();

    /**
     * Checks if a Order with this Number exists.
     * @param orderNumber OrderNumber to be checked.
     * @return true if exists otherwise false
     */
    boolean orderExists(int orderNumber);

    /**
     * Updates a Order.
     * @param order Order with the new informations
     * @return true if success otherwise false
     */
    boolean updateOrder(Order order);
}
