package ch.hslu.appe.usermanagement.persistency;

import java.util.List;

import ch.hslu.appe.usermanagement.exception.CacheOutdatedException;
import ch.hslu.appe.usermanagement.filter.PlaceFilter;
import ch.hslu.appe.usermanagement.model.Place;

/**
 * Place Persistency Interface.
 */
public interface PlacePersistency {

    /**
     * Gets a place by its id.
     * @param id The id
     * @return Returns the place if it exists, otherwise null
     * @throws CacheOutdatedException cache expired
     */
    Place getPlaceById(int id) throws CacheOutdatedException;

    /**
     * Gets a place by its zip code.
     * @param zipCode The zip code
     * @return Returns the place if it exists, otherwise null
     * @throws CacheOutdatedException cache expired
     */
    Place getPlaceByZipCode(int zipCode) throws CacheOutdatedException;

    /**
     * Gets all places.
     * @return Returns a list of all places
     */
    List<Place> getPlaces();

    /**
     * Gets a filtered list of places.
     * @param filter The filter
     * @return Returns a list all places which matches the filter
     */
    List<Place> getPlaces(PlaceFilter filter);

    /**
     * Gets all places within a defined radius.
     * @param origin The origin place
     * @param meters The radius
     * @return Returns a list of all places within the radius
     * @throws CacheOutdatedException cache expired
     */
    List<Place> getPlacesWithinRadius(Place origin, int meters) throws CacheOutdatedException;

    /**
     * Checks whether a place exists.
     * @param zipCode The zip code
     * @return Returns true if it exists, otherwise false
     * @throws CacheOutdatedException cache expired
     */
    boolean placeExists(int zipCode) throws CacheOutdatedException;

    /**
     * Checks whether a place exists.
     * @param place The place
     * @return Returns true if it exists, otherwise false
     * @throws CacheOutdatedException cache expired
     */
    boolean placeExists(Place place) throws CacheOutdatedException;
}
